# -----------------------------------------------------------------------------
# (C) British Crown Copyright 2017-2021 Met Office.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
#
# Plot Fig 1a from "An inter-comparison of the mass budget of the Arctic sea ice in 
# CMIP6 models", Keen et al (2021).
# XX denotes local pathnames etc required 

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import read_budgetfiles as rb
import budget_utils as bu
from SIMIP_Massbudget_DataDic import DataDic # XX A dictionary defining all the filenames etc
from SIMIP_Obs_DataDic import ObsDic # XX A dictionary defining all the filenames etc

# Input data path for models
#---------------------------


path=' ' # Path for model data files


# All CMIP6 models - ordered by sea ice component
ModelNames_CMIP6 = ['NCAR_CESM2_CAM',   # CICE
          'NCAR_CESM2_WACCM', 		# CICE
          'NCC_NorESM2-LM',       	# CICE
          'NCC_NorESM2-MM',       	# CICE
          'MetO_UKESM1',      		# CICE (GSI8.1)
          'MetO_GC3.1_ORCA1',  		# CICE (GSI8.1)    
          'MetO_GC3.1_ORCA025',  	# CICE (GSI8.1)
          'CSIRO_ACCESS-CM2', 		# CICE (GSI8.1)
          'UCLouvain_EC-Earth3-Veg',    # LIM3
          'UCLouvain_EC-Earth3',        # LIM3
          'IPSL_CM6A-LR',          	# LIM3
          'CCCma_CanESM5_x1',          	# LIM2
          'GFDL-CM4',         		# SIS2
          'GFDL-ESM4',        		# SIS2
          'MRI_ESM2.0']         	# MRI


ModelNames=ModelNames_CMIP6
plotfiletag='CMIP6_models'

nskip=4
plotdir='Plot_files/' # Must already exist
plotfile=plotdir+'Fig1a_scycle_ice_area_'+plotfiletag+'+map_inset.png'
plotfile_pdf=plotdir+'Fig1a_scycle_ice_area_'+plotfiletag+'+map_inset.pdf'

# -----------------------------------------------------------------------------

# Obs data
#---------
Obs_path=location

Obs1_tag='HadISST1.2_1990-2009'
filepath=Obs_path+ObsDic[Obs1_tag]['dir']+ObsDic[Obs1_tag]['file']
years,months,obs=rb.read_obsfile(filepath,nskip)
obs=obs/1.0e6
obs1_mn=np.mean(obs, axis=0)
obs1_stdev=np.std(obs, axis=0)

Obs2_tag='HadISST2.2_1990-2009'
filepath=Obs_path+ObsDic[Obs2_tag]['dir']+ObsDic[Obs2_tag]['file']
years,months,obs=rb.read_obsfile(filepath,nskip)
obs=obs/1.0e6
obs2_mn=np.mean(obs, axis=0)
obs2_stdev=np.std(obs, axis=0)


# OSI-SAF data
Obs3_tag='OSI-SAF'
filepath=' '# XX Data location
years,months,obs=rb.read_osi_saf_file(filepath)

# Extract the appropriate period
startyear=1990
endyear=2009
years_index=np.where(np.logical_and (years>=startyear, years<=endyear))
years_subset = years[years_index] 
obs_subset=np.squeeze(obs[years_index,:])

obs3_mn=np.nanmean(obs_subset, axis=0)
obs3_stdev=np.nanstd(obs_subset, axis=0)

# Set up arrays to hold model data for printing out...
nmodels=len(ModelNames)
model_scycles=np.empty([nmodels,12],dtype='float')

# Model data: set up axes, then read in and plot one model at a time
# -----------------------------------------------------------------------------

matplotlib.rcParams.update({'font.size':10})
#fig2 = plt.figure(figsize = (16,14)) 
#fig2 = plt.figure(figsize = (16,10)) # Landscape
#fig2 = plt.figure(figsize = (10,16)) # Portrait

# Scycle of ice area ---------------------------------------------------------------
fig2 = plt.figure(figsize = (8,5)) # Landscape
ax2 = fig2.add_subplot(111) 
ax2.set_title('(a)')
#ax2.set_title('Arctic sea ice area: '+plotfiletag)
ax2.set_ylabel(r'Area (km$^2$ x $10^6$)')
ax2.set_xlabel('Month')


for n,model in enumerate(ModelNames):

	print('Model:'+model)
	filepath=path+DataDic[model]['dir']
	Histfiles=DataDic[model]['HistFiles_ice']

	nensembles=len(Histfiles)
	nyears=20 # 20 years for each hist file
	icearea_scycles=np.empty([nensembles*nyears,12],dtype='float')


	for nensemble,Histfile in enumerate(Histfiles):

		years,months,icearea,icemass,massbudget = rb.read_icefile(filepath+Histfile,nskip)	

		# Extract the required years (1990-2009 mean, to match obs)	
		years_sub,icearea_sub,icemass_sub,massbudget_sub = bu.extract_years(years,

								icearea,icemass,massbudget,1990,2009)
		a=nensemble*nyears
		b=a+nyears
		icearea_scycles[a:b,:] = icearea_sub							

	# Mean and st dev for each model (inlcudes all years and all ensemble members) 
	icearea_scycle_mn=np.mean(icearea_scycles, axis=0)/1.0e6
	icearea_scycle_stdev=np.std(icearea_scycles, axis=0)/1.0e6

	model_scycles[n,:]=icearea_scycle_mn

	# Plot cycle for this model ----------------------------------------

	ax2.plot(months,icearea_scycle_mn, 
            color=DataDic[model]['line_colour2'], 
            linestyle='-',	            
            label=DataDic[model]['label'],
            linewidth=2)
	shading=False 
	if shading:
		ax2.fill_between(months, 
		  icearea_scycle_mn-icearea_scycle_stdev,
		  icearea_scycle_mn+icearea_scycle_stdev,
		  color=DataDic[model]['line_colour2'],
		  alpha=0.25,
		  linestyle='-',
		  linewidth=0.5,
		  antialiased=True)
	error_bars=False
	if error_bars:
		ax2.errorbar(months,icearea_scycle_mn,yerr=icearea_scycle_stdev, 
            	color=DataDic[model]['line_colour2'], 
           	linestyle='-',	            
            	linewidth=2)

# Plot the obs data
ax2.plot(months,obs1_mn, 
    color='black',
    linestyle='dotted',	            
    label=Obs1_tag,
    linewidth=2)
ax2.fill_between(months, 
	obs1_mn-obs1_stdev,
	obs1_mn+obs1_stdev,
	color='black',
	alpha=0.1,
	linestyle='-',
	linewidth=0.5,
	antialiased=True)

ax2.plot(months,obs2_mn, 
    color='black',
    linestyle='dashed',	            
    label=Obs2_tag,
    linewidth=2)
ax2.fill_between(months, 
	obs2_mn-obs2_stdev,
	obs2_mn+obs2_stdev,
	color='black',
	alpha=0.1,
	linestyle='dashed',
	linewidth=0.5,
	antialiased=True)

ax2.plot(months,obs3_mn, 
    color='black',
    linestyle='-',	            
    label=Obs3_tag,
    linewidth=2)
ax2.fill_between(months, 
	obs3_mn-obs3_stdev,
	obs3_mn+obs3_stdev,
	color='black',
	alpha=0.1,
	linestyle='-',
	linewidth=0.5,
	antialiased=True)


ax2.set_xlim([0.5, 12.5])
ax2.legend(loc="best", fontsize = 'x-small')    

addInset = True
if addInset:
   import cartopy.crs as ccrs
   import cartopy.feature as cfeature
   import netCDF4 as nc4
   from mpl_toolkits.axes_grid1.inset_locator import InsetPosition
   #
   datadir=" " # Path for regional mask file
   ORCA025_filename=datadir+"arctic_region_mask_orca025.nc"
   #
   # define hemisphere - using limits to only read relevant rows
   hemisphere = ['NH',[-350,None]]
   hlim = hemisphere[1]
   plot_colours=plt.cm.YlGnBu_r
   #
   # open /read mask file
   fh = nc4.Dataset(ORCA025_filename, mode='r')
   orca025_region = fh.variables['region'][hlim[0]:hlim[1],:]
   orca025_lats = fh.variables['lat'][hlim[0]:hlim[1],:]
   orca025_lons = fh.variables['lon'][hlim[0]:hlim[1],:]
   #
   # mask out land
   orca025_region = np.ma.masked_where(orca025_region<1, orca025_region)
   #
   # add inset axis - will position later
   axins = plt.axes([0, 0, 1, 1], projection=ccrs.NorthPolarStereo())
   axins.set_extent([0.005, 360, 63, 90], crs=ccrs.PlateCarree())
   axins.add_feature(cfeature.LAND, facecolor='0.85')
   axins.add_feature(cfeature.COASTLINE)
   axins.pcolormesh(orca025_lons, orca025_lats, orca025_region,
                     transform=ccrs.PlateCarree(),
                     cmap=plot_colours)
   #
   # set inset location relative to main plot (ax2) in normalized units
   # NB trial and error required here to set these manually I'm afraid
   inset_x = 0.38
   inset_y = 0.215
   inset_size = 0.41
   ip = InsetPosition(ax2, [inset_x - inset_size / 2,
                        inset_y - inset_size / 2,
                        inset_size,
                        inset_size])
   axins.set_axes_locator(ip)

plt.show()
#fig2.savefig(plotfile, bbox_inches="tight", pad_inches=0)
fig2.savefig(plotfile_pdf, dpi=300, bbox_inches="tight", pad_inches=0)
plt.close()


# ---------------That's all folks --------------------




